#include <Foundation.hh>
#include "getCoverage.h"
#include <string>

#include "version.h"


Foundation * tf;

int doSilent = 0;

extern "C"
{


char fn_warnings[MAXFILENAME];
FILE * f_warnings = NULL;

void initializeWarnings()
{
  strcpy(fn_warnings, "getCoverage.warnings");
  unlink(fn_warnings); // This will probably fail

  // Don't try to open the file until there is a warning to print
}

void printWarning(const char * section, char * fmt, ...)
{
  if (f_warnings == NULL)
  {
    f_warnings = fopen(fn_warnings, "a+");
  }

  if (f_warnings || tf)
  {
    char buffer[2048];
    va_list ap;

    va_start(ap, fmt);
    vsnprintf(buffer, sizeof buffer, fmt, ap);
    va_end(ap);

    if (tf)
    {
      tf->getLogger()->getCategory(section).logMsg(buffer, 
                                                  (MessageLevel) DEBUG_WARNING);
    }

    if (f_warnings)
    {
      fprintf(f_warnings, buffer);
      fprintf(f_warnings, "\n");
    }
  }
}



void finalizeWarnings()
{
  if (f_warnings)
  {
    fflush(f_warnings);
    fclose(f_warnings);
    printError("finalizeWarnings", DEBUG_WARNING,
               "Warnings found, see %s for details", fn_warnings);
  }
}

void printError(const char * section, int messageLevel, char * fmt, ...)
{
  if (!doSilent)
  {
    char buffer[2048];
    va_list ap;

    va_start(ap, fmt);
    vsnprintf(buffer, sizeof buffer, fmt, ap);
    va_end(ap);

    if (tf)
    {
      tf->getLogger()->getCategory(section).logMsg(buffer, 
                                                  (MessageLevel) messageLevel);
    }

    fprintf(stderr, buffer);
    fprintf(stderr, "\n");
  }
}

void printInfo(char * fmt, ...)
{
  if (!doSilent)
  {
    va_list ap;

    va_start(ap, fmt);
    vfprintf(stdout, fmt, ap);
    va_end(ap);
  }
}



void logMsg(int level, const char * section, char * fmt, ...)
{
  if (tf)
  {
    char buffer[2048];
    va_list ap;

    va_start(ap, fmt);
    vsnprintf(buffer, sizeof buffer, fmt, ap);
    va_end(ap);

    tf->getLogger()->getCategory(section).logMsg(buffer, (MessageLevel) level);
  }
}

}

int main(int argc, char ** argv)
{
  int retval = 0;

  gRevisionStr[strlen(gRevisionStr)-2] = '\0';
  string version = (string) gVersionStr +
                   (string) " (Build" + (string)(gRevisionStr+9) +
                   (string) ")\n  (" + libSlice_getVersion() +  ")";

  string helptext =
"                                                                               \n\
Display coverage data from .contig and .qual files in a variety of modes at     \n\
specified positions. Output modes include full coverage data, summary data, and \n\
Slice XML representations. Operations can be performed per contig id or for     \n\
all contigs in a .contig file, and over a specifed range of positions.          \n\
                                                                                \n\
  Usage: getCoverage [mode] [coordinates] [options] <prefix>                    \n\
                                                                                \n\
    <prefix> Prefix to .contig and .qual files                                  \n\
                                                                                \n\
    -a|--asmbl_id <asmbl_id> Specify single contig id in multi-contig file      \n\
                             (use 'all' for all contigs in file)                \n\
                                                                                \n\
  Display Modes                                                                 \n\
  -------------                                                                 \n\
  -s|--stats      Calculate a summary of the coverage                           \n\
  -t|--tiling     Calculate full coverage data                                  \n\
  -l|--xml        Display tiling in Slice XML format                            \n\
  -v|--vertical   Display tiling in a greedy vertical representation            \n\
  -H|--horizontal Display tiling in a simple horizontal representation          \n\
  [none]          Display a compact view of the coverage                        \n\
                                                                                \n\
  Coordinate Options                                                            \n\
  ------------------                                                            \n\
  For all modes except the compact view, the default is to calculate coverage   \n\
  information for every position in the contig. For all modes, a range of       \n\
  positions (a cut) can be specified.                                           \n\
                                                                                \n\
  -x {--xg} <pos> Specify ungapped {gapped} position for start of cut           \n\
                  Negative values cut from position to end of contig            \n\
                                                                                \n\
  -y {--yg} <pos> Specify ungapped {gapped} position for end of cut             \n\
                  Negative values specify offset from -x <pos> or end of contig \n\
                                                                                \n\
  -z|--radius <size> Specify a circular cut, use in conjunction with -x {--xg}  \n\
  -r <seqname>  Use the coordinates of the read to specifiy the range           \n\
                                                                                \n\
  Other Options                                                                 \n\
  -------------                                                                 \n\
  -0 Use 0-based coordinates when displaying data (Default for XML)             \n\
  -1 Use 1-based coordiantes when displaying data (Default for other modes)     \n\
                                                                                \n\
  --nogaps      Display ungapped data only                                      \n\
  --offset      Generate read offset file for tiling mode                       \n\
  --nogapqv     Don't Assign gaps the minimum of the flanking qv's in XML mode  \n\
  -M|--missing  Allow missing quality values for reads referenced in contig     \n\
  -G|--gapped   Display gapped coordinates (for tcov and horizontal modes)      \n\
  -u|--unstable Allow the read ids to vary with cut (implies -t)                \n\
  -Q|--silent   Do not display status messages                                  \n\
  -R|--readid   Show the Read index of each read in the idTbl in tiling mode    \n\
  -S|--split    Split output into separate files per contig (implies '-a all')  \n\
  -i|--index    Build or use an index on the quality values                     \n\
  -C|--circular Assume reads that extend beyond the consensus are circular      \n\
                *NOT* necessary for reads with negative offsets.                \n\
                                                                                \n\
  -o <prefix>   Specify prefix for results or the directory for Split files     \n\
                (Default prefix to contig file and ./)                          \n\
                                                                                \n\
  -c <mode>     Recall the consensus in specified ambiguity mode for stats file \n\
                (1=No ambiguity, 2=Minimal Ambiguity, 3=Annotation, 4=Conic)    \n\
                                                                                \n\
Please see http://intranet/software_docs/getCoverage.html for more details.     \n\
";
  string dependencies = "";

  tf = NULL;

  try
  {
    tf = new Foundation(version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    string asmbl_id;
    string outputprefix = "";

    // Tiling options
    int doTiling         = 0;
    int doUnstable       = 0;
    int doStats          = 0;
    int print_offset     = 0;
    int doBasesCoverage  = 0;
    int doCalcConsensus  = 0;
    int doSplit          = 0;
    int doVertical       = 0;
    int doHorizontal    = 0;

    // Cut options
    int ungapped_pos = -1;
    int gapped_pos   = -1;

    int radius       = 5;
    int ungapped_end = 0;
    int gapped_end   = 0;

    // Other options
    int doXML        = 0;
    int doNoGaps     = 0;
    int doZeroBased  = 0;
    int doOneBased   = 0;

    int showReadId   = 0;
    int doGapped     = 0;
    int allowMissing = 0;
    int doGapQV      = 1;
    int defaultQV    = -1;
    int doIndex      = 0;
    int doCircular   = 0;

    string seqname_range  = "";

    tf->getOptions()->addOptionResult("t|tiling", &doTiling, 
                                      "Calculate tiling and write to file");

    tf->getOptions()->addOptionResult("s|stats", &doStats,
                                      "Calculating basic coverage stats and write to file");

    tf->getOptions()->addOptionResult("l|xml", &doXML,
                                      "Print Slice XML of contig or a cut");

    tf->getOptions()->addOptionResult("x|ugap=i", &ungapped_pos, 
                                      "Ungapped assembly position for cut");
    
    tf->getOptions()->addOptionResult("xg|g|gap=i", &gapped_pos, 
                                      "Gapped assembly position for cut");

    tf->getOptions()->addOptionResult("y|yugap=i", &ungapped_end, 
                                      "Ungapped assembly position for cut end");
    
    tf->getOptions()->addOptionResult("yg|ygap=i", &gapped_end, 
                                      "Gapped assembly position for cut end");
    
    tf->getOptions()->addOptionResult("z|radius=i", &radius,
                                      "Radius for cut");

    tf->getOptions()->addOptionResult("0", &doZeroBased, 
                                      "Display 0-based coordinates (XML Default)");

    tf->getOptions()->addOptionResult("1", &doOneBased, 
                                      "Display 1-based coordinates (default)");

    tf->getOptions()->addOptionResult("a|asmbl_id=s", &asmbl_id, 
                                      "Assembly Id to process (use 'all' for all in .contig file)");

    tf->getOptions()->addOptionResult("b|basescoverage", &doBasesCoverage,
                                      "Calculate bases coverage for stats");

    tf->getOptions()->addOptionResult("c|consensus=i", &doCalcConsensus,
                                      "Recompute consensus in stats (0=off 1=base 2=ambiguity 3=annotation)");

    tf->getOptions()->addOptionResult("G|gapped", &doGapped, 
                                      "Display gapped coordinates in tcov");

    tf->getOptions()->addOptionResult("nogaps", &doNoGaps, 
                                      "Don't output slices with gap as consensus");

    tf->getOptions()->addOptionResult("Q|silent", &doSilent, 
                                      "Don't display status messages");

    tf->getOptions()->addOptionResult("R|readid", &showReadId, 
                                      "Show read id for each read in idTbl");

    tf->getOptions()->addOptionResult("S|split", &doSplit, 
                                      "Split results by contig id (Implies -a all)");

    tf->getOptions()->addOptionResult("u|unstable", &doUnstable, 
                                      "Allow for unstable read ids in tiling mode");

    tf->getOptions()->addOptionResult("M|missing", &allowMissing,
                                      "Allow missing quality values");
        
    tf->getOptions()->addOptionResult("o|outdir=s", &outputprefix,
                                      "Specify the directory for output");

    tf->getOptions()->addOptionResult("offset", &print_offset, 
                                      "Generate read offset file for tiling");

    tf->getOptions()->addOptionResult("p|gapqv!", &doGapQV, 
                                      "Assign Quality Values to gaps");

    tf->getOptions()->addOptionResult("q|defaultqv=i", &defaultQV, 
                                      "Quality Value to assign when missing");

    tf->getOptions()->addOptionResult("v|vertical", &doVertical, 
                                      "Calculate vertical tiling for realigner");

    tf->getOptions()->addOptionResult("H|horizontal", &doHorizontal, 
                                      "Calculate horizontal tiling (cutAsm)");

    tf->getOptions()->addOptionResult("r=s", &seqname_range, 
                                      "Use range of sequence to specify range");

    tf->getOptions()->addOptionResult("i|index", &doIndex, 
                                      "Build or load index on quality values");

    tf->getOptions()->addOptionResult("C|circular", &doCircular, 
                                      "Assume reads that extend beyond the consensus are circular");



    tf->handleStandardOptions();

    string prefix = tf->getOptions()->getNextOtherData();
    string::size_type dotpos;
    if (prefix.empty())
    {
      cerr << "You must specify a prefix to the .contig and .qual files" << endl
           << "Try 'getCoverage --help' for more information" << endl;

      retval = 1;
      throw ExitProgramNormally();
    }
    else if ((dotpos = prefix.find(".contig")) != prefix.npos)
    {
      // Drop the .contig from the filename
      prefix.erase(dotpos, prefix.length());
    }

    // Determine if 0 or 1 based coordinates should be used.
    int coordinateOffset;
    if (doXML)
    {
      //XML is 0-based by default
      coordinateOffset = (doOneBased) ? 1 : 0;
    }
    else
    {
      //Everything else is 1-based
      coordinateOffset = (doZeroBased) ? 0 : 1;
    }

    // Use NULL to indicate that it was not specified on command line
    int *pGappedPos   = tf->getOptions()->hasOption("gap")
                        ? &gapped_pos : NULL;

    int *pUngappedPos = tf->getOptions()->hasOption("ugap")
                        ? &ungapped_pos : NULL;

    int *pGappedEnd   = tf->getOptions()->hasOption("ygap") 
                        ? &gapped_end : NULL;

    int *pUngappedEnd = tf->getOptions()->hasOption("yugap") 
                        ? &ungapped_end : NULL;

    enum DisplayMode displayMode = COV;

    if      (doTiling)     { displayMode = TCOV; }
    else if (doUnstable)   { displayMode = TCOV_UNSTABLE; }
    else if (doVertical)   { displayMode = VERTICAL; }
    else if (doStats)      { displayMode = STATS; }
    else if (doHorizontal) { displayMode = HORIZONTAL; }
    else if (doXML)        { displayMode = XML; }


    // See BITS #21917 for a rational of this
    // The trick is if there is a trailing slash, assume asmbl_id specific file should be created

    bool hasSlash = !outputprefix.empty() && outputprefix[outputprefix.length()-1] == '/';
    bool defaultStdout = (displayMode == HORIZONTAL || displayMode == XML || displayMode == COV);

    string inputdir = prefix;
    string::size_type slash = inputdir.rfind('/');

    if (slash == inputdir.npos) 
    { 
      // "" => "./"
      inputdir = "./"; 
    }                          
    else
    { 
      // "blah/somthing/file" ==> "blah/something/"
      inputdir.erase(slash+1, inputdir.size()); 
    } 

    if (doSplit)
    {
      if (outputprefix.empty())                     
      { 
        // "" => inputdir
        outputprefix = inputdir; 
      }
      else if (outputprefix[0] != '-' && !hasSlash) 
      { 
        // "something" => "something/"
        outputprefix += "/"; 
      }
    }
    else 
    {
      if (hasSlash)
      {
        printError("Coverage", DEBUG_ERROR,
                   "-o must specifiy a filename prefix");
        retval = 1;
        throw ExitProgramNormally();
      }
      
      if (asmbl_id == "all")
      {
        if (defaultStdout)
        {
          printError("Coverage", DEBUG_ERROR,
                     "Must use split mode (-S) for multiple asmbl_ids with chosen display mode");
          retval = 1;
          throw ExitProgramNormally();
        }
        else if (outputprefix.empty())
        {
          // "" => "blah/something/file"
          outputprefix = prefix;
        }
      }
      else if (!defaultStdout && outputprefix.empty())
      {
        // "" => "blah/something/"
        outputprefix = inputdir;
      }
    }

    if (defaultQV > -1)    { allowMissing = defaultQV; }
    else if (allowMissing) { allowMissing = 0;         }
    else                   { allowMissing = -1;        }

    const char * oprefix = (outputprefix.empty()) ? NULL : outputprefix.c_str();

    const char * cseqname = (seqname_range == "") 
                           ? NULL : seqname_range.c_str();

    //fprintf(stderr, "oprefix is %s\n", oprefix ? oprefix : "(NULL)");
    //return 0;

    initializeWarnings();
    setDoGapQV(doGapQV);

    retval = getCoverage(prefix.c_str(), asmbl_id.c_str(),
                         pUngappedPos, pGappedPos, radius,
                         pUngappedEnd, pGappedEnd, coordinateOffset,
                         displayMode, print_offset, 
                         doBasesCoverage, doCalcConsensus,
                         !doNoGaps, oprefix, doSplit, showReadId,
                         doGapped, allowMissing, doGapQV, cseqname,
                         doIndex, doCircular);

  }
  catch (const ExitProgramNormally & e)
  {

  }
  catch (const Exception & e)
  {
    cerr << e << endl;

    retval = 100;
  }

  finalizeWarnings();

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 101;
  }

  return retval;
}
