/*
  getCoverage.h

  $Id: getCoverage.h,v 1.1.1.1 2005/08/04 14:32:29 mcschatz Exp $
*/

#ifndef __GETCONTIG_H__
#define __GETCONTIG_H__ 1


#ifdef __cplusplus
extern "C"
{
#endif

#include "config.h"

#ifndef _LARGE_FILES
#define ftello ftell
#define fseeko fseek
#endif

#define MAXLINE     512 
#define MAXSEQNAME  255
#define MAXFILENAME 255

#include <stdarg.h>
#include <errno.h>
#include <stdio.h>
#include <math.h>
#include "Slice.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

typedef struct COORDINATES
{
  int start;
  int end;
  int ugInd;
  int coordinateOffset;
} Coordinates;

typedef short seqlen_t;
typedef char  qvdata_t;

typedef struct OFFSET
{
  char *id;  /* read id */
  off_t offset;
  seqlen_t qvseqlen;
} Offset;

struct CONTIG_SEQUENCE 
{
  char *id;             /* Sequence name                                    */
  char * qvseq;
  seqlen_t qvseqlen;
  off_t qvoffset;
  off_t f_offset;
  char *seq;            /* To hold letters of the sequence                  */
  int len;              /* Sequence length                                  */
  char rc;              /* The sequence has been reversed complemented      */
  int offset;
  int seq_l;   
  int seq_r;   
  int asm_l;   
  int asm_r;   
  int *gapInd;           /* gap index for cseq[i]->seq                       */
  int numOfGaps;
};


typedef struct TILING_DATA 
{
  unsigned int **idcov;
  unsigned short *cdc;
  unsigned short *dcov;
  int numOfSeqs;
  int maxDepth;
} tilingData;

enum DisplayMode
{
  COV = 0,
  TCOV,
  TCOV_UNSTABLE,
  XML,
  STATS,
  VERTICAL,
  HORIZONTAL,
};

int getCoverage(const char * prefix, const char * asmbl_id,
                int * ungapped_pos, int * gapped_pos, int radius,
                int * ungapped_end, int * gapped_end, int coordinateOffset,
                enum DisplayMode displayMode, int print_offset, int doBasesCoverage, 
                int doCalcConsensus, int allowGaps, const char * output_prefix, 
                int doSplit, int showReadId, int doGapped, char allowMissing,
                int doGapQV, const char * seqname_range, char useIndex,
                char assumeCircular);


void readContig(FILE *f, struct CONTIG_SEQUENCE **contig, int *numOfSeqs);
char readSeqs(int * numOfSeqs, struct CONTIG_SEQUENCE ***seq, 
              struct CONTIG_SEQUENCE * contig, Coordinates * coords, 
              const char * seqname_range, char assumeCircular,
              char loadAll);

void loadSeq(struct CONTIG_SEQUENCE *bcseq);

void setRangeCoordinates(struct CONTIG_SEQUENCE * bcseq,
                         struct CONTIG_SEQUENCE * contig,
                         Coordinates * coords);


void indexSeqIDs(FILE *f, char *file, Offset **qvIndex,
                 int *numOfreads, char useIndex);

void alignSeqs(struct CONTIG_SEQUENCE **bcseq,
               struct TILING_DATA *tdata,
               int numOfSeqs, int contigLen);

int loadTilingChunk(int alignStart,
                    struct TILING_DATA *tdata,
                    struct CONTIG_SEQUENCE **bcseq,
                    struct CONTIG_SEQUENCE * contig,
                    char doLoadQV);

void setDoGapQV(int doGapQV);
void parseQual(Offset *index,
               int numOfQVSeqs,
               int numOfSeqs,
               struct CONTIG_SEQUENCE **bcseq,
               char * contig_id,
               char allowMissing,
               int doGapQV);

void setQualFile(FILE *fqual);
void closeQualFile();
void loadQual(struct CONTIG_SEQUENCE *bcseq, char * contig_id);


int binsearchOffset(char *id, Offset *seq,int num);

void convertCoordinates(struct CONTIG_SEQUENCE *contig,
                        int * ungapped_pos, int * gapped_pos, int radius,
                        int * ungapped_end, int * gapped_end,
                        const char * seqname_range,
                        Coordinates * coords);

void printTotalCov(struct CONTIG_SEQUENCE *contig,
                   struct CONTIG_SEQUENCE **bcseq,
                   tilingData *td, 
                   int allowGaps,
                   Coordinates * coords,
                   const char * output_prefix,
                   const char * prefix,
                   int print_asmbl_id,
                   int print_offset,
                   int showReadId,
                   int doGapped);

void printStats(struct CONTIG_SEQUENCE *contig,
                struct CONTIG_SEQUENCE **bcseq,
                tilingData *td, 
                int allowGaps,
                Coordinates * coords,
                const char * output_prefix,
                const char * prefix,
                int print_asmbl_id,
                int doBasesCoverage,
                int doCalcConsensus);


void printCov(struct CONTIG_SEQUENCE *contig,
              struct CONTIG_SEQUENCE **bcseq,
              tilingData *td, 
              int allowGaps,
              Coordinates * coords,
              const char * output_prefix);

void printXML(struct CONTIG_SEQUENCE *contig,
              struct CONTIG_SEQUENCE **bcseq,
              tilingData * td,
              int allowGaps,
              Coordinates * coords,
              const char * output_prefix);


void printVertical(struct CONTIG_SEQUENCE *contig,
                   struct CONTIG_SEQUENCE **bcseq,
                   tilingData *td, 
                   Coordinates * coords,
                   const char * output_prefix,
                   const char * prefix);

void printHorizontal(struct CONTIG_SEQUENCE *contig,
                     struct CONTIG_SEQUENCE **bcseq,
                     tilingData *td, 
                     int showGapped,
                     Coordinates * coords,
                     const char * output_prefix);


int hasOverlap(int rangeStart, int rangeEnd, 
               int seqOffset, int seqLen, int contigLen);



// memory allocation routines
void * newmem2(int number, int size, char *file, int line);
void * pool_malloc(int size);
void   pool_flush();

// free data
void freeCovData    (tilingData *td);
void freeContigData (struct CONTIG_SEQUENCE *contig);
void freeSeqData    (struct CONTIG_SEQUENCE **bcseq, int numOfSeqs);
void freeOffset     (Offset *index, int numOfSeqs);

FILE *fOpen(const char *file, const char *format);

#define DEBUG_FATAL   10
#define DEBUG_ERROR   20
#define DEBUG_WARNING 30
#define DEBUG_INFO    40
#define DEBUG_LOW     50
#define DEBUG_MEDIUM  60
#define DEBUG_HIGH    70

void logMsg(int level, const char * section, char * fmt, ...);
void printError(const char * section, int level, char * fmt, ...);
void printInfo(char * fmt, ...);
void printWarning(const char * section, char * fmt, ...);


#ifdef __cplusplus
}
#endif

#endif /* ifndef __GETCONTIG_H__ */

