#include "getCoverage.h"

#define POOLBLOCKSIZE 1024*1024

struct poolNode
{
  char * block;
  int offset;
  int bytesRemaining;
  int size;

  struct poolNode * next;
};

static struct poolNode * poolHead = NULL;

// Parcel memory from big byte buckets. Return all memory with 8 byte alignment
// to silence the bus errors on Alpha and Solaris.
void * pool_malloc(int requestedSize)
{
  void * retval = NULL;

  int remainder = requestedSize % 8;

  if (remainder)
  {
    // Make sure the next block is 8 byte aligned
    requestedSize += 8 - remainder;
  }

  if (poolHead == NULL || requestedSize > poolHead->bytesRemaining)
  {
    int blockSize = POOLBLOCKSIZE;
    struct poolNode * newHead;

    if (requestedSize > blockSize)
    {
      blockSize = requestedSize;
    }

    newHead = (struct poolNode *) newmem2(1, sizeof(struct poolNode),__FILE__,__LINE__);

//    fprintf(stderr, "Allocated a new %d byte block\n", blockSize);

    newHead->block = (char *) newmem2(1, blockSize, __FILE__,__LINE__);
    newHead->bytesRemaining = blockSize;
    newHead->size = blockSize;
    newHead->offset = 0;

    newHead->next = poolHead;
    poolHead = newHead;
  }

  retval = poolHead->block + poolHead->offset;

  poolHead->bytesRemaining -= requestedSize;
  poolHead->offset += requestedSize;

  return retval;
}


void pool_flush()
{
  struct poolNode * next;

  int size = 0;
  int blocks = 0;

  while(poolHead)
  {
    size += poolHead->size;
    blocks++;

    next = poolHead->next;
    free(poolHead->block);
    free(poolHead);
    poolHead = next;
  }

  /*
  fprintf(stderr, "Blocks: %d poolsize: %.fMb\n", 
          blocks, ((float)(size))/(1024*1024));
  */
}


