#include "SliceEditor.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = "Usage: editSlice [OPTIONS] contig.corrected contig.orig.slice";

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);

    SliceEditor::handleOptions(tf);
    SliceEditor manager;

    string editFile;
    
    // Read the input filename
    if ((editFile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an edit file", "editSlice");
    }

    manager.processEditFile(editFile);

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "editSlice");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);

  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
