#ifndef CONTIGSEQUENCE_HH 
#define CONTIGSEQUENCE_HH 1

#include "Foundation.hh"
#include "RequestManager.hh"

#include <vector>
#include <deque>

typedef deque<char> QualSequence;

class ContigSequence
{
public:
  ContigSequence(ReadData * rd, 
                 long gindex, 
                 int readCount);
  ~ContigSequence();

  const string & getSeqname() const;
  const char * getNuc(long gindex);
  const char * getQual(long gindex);
  const char * getReadId();

  void addSlice(char base, char qv);
  void addSequence(const string & bases, const string & quals);

  char & operator[](long gindex);

  int rightTwiddle(long gindex, int & twiddleDistance, char cons);
  int leftTwiddle(long gindex, int & twiddleDistance, char cons);

  void erase(long gindex);
  char base(long gindex);
  int qv(long gindex) const;

  float getAvgQV(long offset, int count) const;
  float getAvgQV3Trim(int count) const;

  void dump();
  void dumpRange(long gindexstart, long gindexend);
  void printReadData(ostream & out);
  void printContigSequence(ostream & out, long asm_l=1, long asm_r=1);

  void adjustOffset(long gindex, int delta);

  long getRightEnd() const;
  long getOffset() const;
  string getRightTrim();
  string getLeftTrim();
  int get3TrimLength();

  int extendRight(const string & extendRight);
  int extendLeft(const string & extendLeft);

  int trimReadRight(long gindex);
  int trimReadLeft(long gindex);

  int getReadCount() const;
  void insertGap(long gindex);

  bool isReverseCompliment() const;

  long stripLeftGaps();
  long stripRightGaps();

private:
  ReadData * m_readData;

  void swap(long a, long b);
  int extractFirstQuality(string & qualstring, bool doErase);

  string m_nuc;
  QualSequence m_qual;

  string m_rightTrim;
  string m_rightTrimQual;

  string m_leftTrim;
  string m_leftTrimReversed;
  string m_leftTrimQual;

  int m_readCount;

  long m_offset;
  int m_length;
};

#endif
