#include "SliceOptimizer.hh"
#include "ContigOptimizer.hh"

SliceOptimizer::SliceOptimizer() : RequestManager("SliceOptimizer")
{
  m_log.debugL("Contructed SliceOptimizer");
  m_log.setMessageLevel(MESSAGE_LEVEL_DEBUG_MEDIUM);
  m_contig = NULL;
}

int SliceOptimizer::s_verbose(0);


SliceOptimizer::~SliceOptimizer()
{
  if (m_contig) delete m_contig;
  m_log.debugL("Destructed SliceOptimizer");
}

void SliceOptimizer::addSliceRange(long & index, long & gindex)
{
  m_contig = new Contig(index, gindex, m_readManager, s_verbose);

  index = gindex = 0L;
}

int SliceOptimizer::processContigFile(const string & infile, ostream & error)
{
  ifstream * cf = FileSystem::openInputFile(infile);

  while(*cf)
  {
    m_contig = new Contig(0, 0, m_readManager, s_verbose);
    m_contig->loadContig(cf);

    ContigOptimizer optimizer(s_verbose);
    optimizer.optimize(m_contig);

    m_contig->printContig(m_out, CONTIG);
  }

  delete cf;

  return 0;
}



void SliceOptimizer::addSlice(SliceData * slice)
{
  m_contig->addSlice(slice);
}

void SliceOptimizer::addRequest(ConsensusRequest * request)
{
  RequestManager::addRequest(request);

  printXMLHeader(request);
}


int SliceOptimizer::printResults()
{
  ContigOptimizer optimizer(s_verbose);
  optimizer.optimize(m_contig);

  m_contig->printContig(m_out);
  
  m_out << "</Request>" << endl;

  return m_errorCount;
}

void SliceOptimizer::handleOptions(Foundation * tf)
{
  tf->getOptions()->addOptionResult("v|verbose", &s_verbose,
                                    "Be verbose when reporting");

  RequestManager::handleOptions(tf);
}

