#include "SliceSplitter.hh"

SliceSplitter::SliceSplitter() : RequestManager("SliceSplitter")
{
  m_log.debugL("Contructed SliceSplitter");
  m_log.setMessageLevel(MESSAGE_LEVEL_DEBUG_MEDIUM);

  m_rangeCount = -1;
  m_currentOut = NULL;
  m_firstSlice = true;

  if (s_partitionString != "")
  {
    loadPartitions(s_partitionString);
  }

  if (s_partitionFile != "")
  {
    loadPartitionFile();
  }

  if (m_partitionList.empty())
  {
    throw Exception("No partition points specified", 
                        "SliceSplitter::SliceSplitter");
  }

  m_partitionList.sort();
}

string SliceSplitter::s_partitionString("");
string SliceSplitter::s_partitionFile("");

SliceSplitter::~SliceSplitter()
{
  m_log.debugL("Destructed SliceSplitter");
}

void SliceSplitter::getNextOutfile()
{
  if (m_currentOut)
  {
    *m_currentOut << "  </SliceRange>" << endl
                  << "</Request>"      << endl;
    delete m_currentOut;
  }

  m_rangeCount++;

  char buffer[1024];
  snprintf(buffer, sizeof(buffer), "_%d.slice", m_rangeCount);

  string prefix;

  if (m_outfile == "")
  {
    prefix = m_infile;

    string::size_type dot = prefix.rfind('.');
    if (dot != string::npos)
    {
      prefix.erase(dot, prefix.size());
    }
  }
  else
  {
    prefix = m_outfile;
  }

  string outfile = prefix + buffer;

  m_currentOut = openOutputFile(outfile);

  printXMLHeader(m_requestData, m_currentOut);

  *m_currentOut << "  <ReadCollection>" << endl;
  m_readManager->resetReadIterator();
  
  ReadData * rd;
  while ( (rd = m_readManager->getNextRead()) != NULL)
  {
    rd->printXML(*m_currentOut);
  }

  *m_currentOut << "  </ReadCollection>" << endl;
}

void SliceSplitter::addSlice(SliceData * slice)
{
  RequestManager::addSlice(slice);

  if (m_firstSlice 
      || (!m_partitionList.empty() && slice->getIndex() == m_partitionList.front()))
  {
    if (m_firstSlice) 
    { 
      m_firstSlice = false; 
      if (slice->getIndex() == m_partitionList.front())
      {
        // If the first slice is the first partition, silently ignore
        m_partitionList.pop_front();
      }
    }
    else { m_partitionList.pop_front(); }

    getNextOutfile();
    *m_currentOut << "  <SliceRange Index=\"" << slice->getIndex()
                  << "\" Gindex=\""           << slice->getGindex()
                  << "\">" << endl;
  }

  slice->printXML(*m_currentOut);

  delete slice;
}


int SliceSplitter::printResults()
{
  if (m_currentOut)
  {
    *m_currentOut << "  </SliceRange>" << endl
                  << "</Request>"      << endl;
  }

  return m_errorCount;
}

void SliceSplitter::handleOptions(Foundation * tf)
{
  tf->getOptions()->addOptionResult("p=s", &s_partitionString,
                                    "Specify list of partition elements");

  tf->getOptions()->addOptionResult("f=s", &s_partitionFile,
                                    "Specify file of partition elements");

  RequestManager::handleOptions(tf);

}

void SliceSplitter::addPartition(int partition)
{
  //cerr << "spliting at " << partition << endl;
  m_partitionList.push_back(partition);
}

void SliceSplitter::loadPartitions(const string & partitionlist)
{
  // Split the partitionlist on ',', load each into the master list

  std::string::size_type index = 0;
  std::string::size_type splitIndex;
  int gappos;

  while ((splitIndex = partitionlist.find(',', index)) != partitionlist.npos)
  {
    gappos = atoi(partitionlist.substr(index, splitIndex - index).c_str());
    addPartition(gappos);
    index = splitIndex + 1;
  }

  gappos = atoi(partitionlist.substr(index, 
                               partitionlist.length() - index + 1).c_str());
  addPartition(gappos);

}

void SliceSplitter::loadPartitionFile()
{
  ifstream * pFile = FileSystem::openInputFile(s_partitionFile);

  if (pFile)
  {
    int partition;

    while (*pFile)
    {
      (*pFile) >> partition;
      if (*pFile)
      {
        addPartition(partition);
      }
    }
  }
}
